package hu.stunts.scr.apps;

import hu.stunts.scr.utils.InfoFormatter;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.List;
import java.util.ListIterator;

public abstract class AbstractScoreboardGenerator {
	
	List<? extends BaseReplay> replays;
	InfoFormatter rFormat;
	
	public AbstractScoreboardGenerator(List<? extends BaseReplay> replayList,InfoFormatter rf) {
		replays = replayList;
		rFormat = rf;
	}
	
	public String[][] generateHIGStrings (boolean useCorrectedTimes) {
		String[][] output = null;
		if (replays.size() > 0) {
			output = new String[replays.size()][8];
			int i = 0;
			for (BaseReplay replay : replays) {
				output[i][0] = replay.getRacer();
				output[i][1] = rFormat.formatCarName(replay.getCar());
				output[i][2] = InfoFormatter.formatOpponentName(replay.opponentIndex());
				if (replay.opponentIndex() != InfoFormatter.CLOCK)
					output[i][3] = rFormat.formatCarAbbr(replay.getCar());
				else
					output[i][3] = "";
				if (useCorrectedTimes)
					output[i][4] = InfoFormatter.formatLaptime(Double.toString(replay.toSeconds(replay.computeTime())));
				else
					output[i][4] = InfoFormatter.formatLaptime(Double.toString(replay.toSeconds(replay.computeDuration(true))));
				if (replay.autoTransmission())
					output[i][5] = "A";
				else
					output[i][5] = "M";
				output[i][6] = InfoFormatter.formatTimeGap(Double.toString(replay.toSeconds(replay.compareTo(replays.get(0)))));
				output[i][7] = replay.getTrack();
				i++;
			}
		}
		return output;
	}
		
	
	public void writeHIG (File dest, boolean correctTimes){
		try {
			File newFile = dest;
			if (newFile.exists())
					newFile.delete();
			newFile.createNewFile();
			RandomAccessFile higFile =
				new RandomAccessFile(newFile, "rw");
			BaseReplay currReplay;
			ListIterator<? extends BaseReplay> it = replays.listIterator();
			for (int i = 0; i < 7; i++) {
				if (it.hasNext()) {
					currReplay = it.next();
					for (int j = 0; j < 16; j++) { //Racer
						if (j < currReplay.getRacer().length())
							higFile.writeByte(currReplay.getRacer().charAt(j));
						else
							higFile.writeByte(0);
					}
					higFile.writeByte(0);
					for (int j = 0; j < 24; j++) { //Car
						String carName = rFormat.formatCarName(currReplay.getCar());
						if (j < carName.length())
							higFile.writeByte(carName.charAt(j));
						else
							higFile.writeByte(0);
					}
					higFile.writeByte(0);
					for (int j = 0; j < 7; j++){ //Opponent
						if (currReplay.opponentIndex() != 0) {
							String oppCarID = currReplay.opponentCarID();
							String opp = InfoFormatter.formatOpponentAbbr(currReplay.opponentIndex())
							+ "/" + oppCarID.charAt(0)
							+ rFormat.formatCarAbbr(currReplay.getCar());
							if (j < opp.length())
								higFile.writeByte(opp.charAt(j));
							else
								higFile.writeByte(0);
						}
						else
							higFile.writeByte(0);
					}
					higFile.writeByte(0);
					int time;
					if (correctTimes)
						time = (int)Math.round(currReplay.computeTime(true));
					else
						time = (int)Math.round(currReplay.computeDuration(true));
					higFile.writeByte(time -(int)(time/0x100)*0x100);
					higFile.writeByte((int)time/0x100);
				} 
				else { //empty entry
					for (int j = 0; j < 40; j++)
						higFile.writeByte('.');
					higFile.writeByte(0);
					higFile.writeByte(0);
					for (int j = 0; j < 2; j++)
						higFile.writeByte('.');
					higFile.writeByte('/');
					for (int j = 0; j < 4; j++)
						higFile.writeByte('.');
					higFile.writeByte(0);
					higFile.writeByte(0xFF);
					higFile.writeByte(0xFF);
				}
			} 

		} catch (IOException e) {
			System.err.println ("Writing hig failed!");
			e.printStackTrace();
		} 
	}

}
