package hu.stunts.scr.apps;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import hu.stunts.scr.exceptions.MissingReferenceTrackException;
import hu.stunts.scr.exceptions.NoScoreboardEntriesException;
import hu.stunts.scr.ui.*;
import hu.stunts.scr.utils.InfoFormatter;
import hu.stunts.scr.utils.enums.*;

public class JRPLInfo extends JFrame {
	/**
	 * 
	 */
	private static final long serialVersionUID = 5424791800362050939L;
	
	//General variables
	
	private static final int MAX_INFO_FIELDS = 12;
	private static final int INFO_TABLE_OFFSET = 2; // offset for array indices in rplInfo output
	private static final int TAB_RPLINFO = 0;
	private static final int TAB_RACEVIEW = 1;
	private boolean isZSC = false; // ZSC bonuses checkbox variable
	private File workingDir = new File(".");
	private String rplPath = "";
	private BaseReplay loadedReplay;
	private RplDirectory loadedDir;
	private int rplDirIndex = 0; // current position while cycling through dir
	private InfoFormatter riFormatter = null;
	
	//Interface elements
	
	
	private JTabbedPane tabs;
	
	private JMenu[] menus = {
			new JMenu("File"),
			new JMenu("Help")
	};
	
	private JMenuItem[][] menuItems = {
			{//file
				new JMenuItem("Load Replay..."),
				new JMenuItem("Open Directory..."),
				new JMenuItem("Exit")
			},
			{//help
				new JMenuItem("About")
			}
	};
	
	//see menuListeners for the listeners of these menu items.

	private JButton
	    bLoad = new JButton("Load .RPL"),
	    bOpenDir = new JButton("Open dir"),
		bRA = new JButton("Log keystrokes"),
		bPrev = new JButton("<"),	
		bNext = new JButton(">"),	
		bRaceFromInfo = new JButton("Open RPLInfo dir");
	private JCheckBox 
		zscCheck = new JCheckBox("Use ZSC Bonuses", false);

	private JLabel
		lDirIndex = new JLabel("0"),
		lDirCount = new JLabel("0");
	
	private InfoGridPanel gpInfo;
	private InfoGridPanel gpRA;
	private TextDump pConsole;
	
	//Resources
	
	private String[] infoHeaders = {
			"File name",
			"Directory",
			"Game Version",
			"Track",
			"Car",
			"Colour",
			"Transmission",
			"Opponent",
			"Opponent Car",
			"Matches reference .TRK?",
			"Appears finished?",
			"Lap duration (s)",
			"Laptime with penalty (s)",
			"Corrected time (s)"
	};
	private String[] infoFields = new String[infoHeaders.length];
	private String[] raHeaders = {
			"Accelerate",
			"Brake",
			"Steer Right",
			"Steer Left",
			"Shift Up",
			"Shift Down",
			"No Input",
			"Unused"
	};
	private String[] raFields = new String[raHeaders.length];
	

	
	//Listeners
	
	private ActionListener onAbout = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			FrameWithClose frame = new FrameWithClose("About");
		    TextDump raNotes = new TextDump("About JRPLInfo", 
		    		"JRPLInfo 0.1\n\n" +
		    		"A Southern Cross Racing Developments Software\n" +
		    		"Developed by Daniel David (alias Duplode)\n" +
		    		"Licensed under the GPL v3\n" +
		    		"\"By the community, for the community\"\n\n" +
		    		"Visit the Stunts Portal:" + 
		    		"http://stunts.hu\n\n"+
		    		"March 2011");
		    frame.mainBox().add(raNotes);
		    frame.setVisible(true);
		}
	};
	
	private ActionListener onExit = new ActionListener() {
    	public void actionPerformed(ActionEvent e) {
    		stop();
    		dispose();
    	}
	};
	
	private ActionListener onZSCCheck = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			isZSC = !isZSC;
		}
	};

	private ActionListener onLoad = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			pConsole.setDumpField("");
			JFileChooser c = new JFileChooser(workingDir);
			StuntsFcExtensionFilter fcf = new StuntsFcExtensionFilter(new String[] {"RPL"}, "Replay files" );
			c.addChoosableFileFilter (fcf);
			c.setFileFilter(fcf);
			c.setDialogTitle("Load Replay File");
			int rVal = c.showOpenDialog(JRPLInfo.this);
			if(rVal == JFileChooser.APPROVE_OPTION) {
				File f = c.getSelectedFile();
				workingDir = f.getParentFile();
				loadRplFileInfo(f);
				if (loadedDir == null || loadedDir.getDirPath() != f.getParent()) {
					loadRplDir(f.getParentFile());
					if (loadedDir != null && loadedDir.getRplCount() > 0) {
						rplDirIndex = loadedDir.searchRplList(f.getName());
						lDirIndex.setText(Integer.toString(rplDirIndex + 1));
						bPrev.setEnabled(true);
						bNext.setEnabled(true);
						bRaceFromInfo.setEnabled(true); //activation is controlled by the RPLInfo panel.
					}
					else {
						lDirIndex.setText("0");
						bPrev.setEnabled(false);
						bNext.setEnabled(false);
						bRaceFromInfo.setEnabled(false);
					}
				}
			}
		}
	};
	
	private ActionListener onOpenDir = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			pConsole.setDumpField("");
			JFileChooser c = new JFileChooser(workingDir);
			c.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			c.setDialogTitle("Open Directory");
			int rVal = c.showOpenDialog(JRPLInfo.this);
			if(rVal == JFileChooser.APPROVE_OPTION) {
				File d = c.getSelectedFile();
				workingDir = d; //changes workingDir even if dir is empty.
				rplDirIndex = 0;
				loadRplDir(d);
				if (loadedDir.getRplCount() > 0) {
					bPrev.setEnabled(true);
					bNext.setEnabled(true);
					bRaceFromInfo.setEnabled(true);
					loadRplFileInfo(new File(d.getPath() + File.separator + loadedDir.getFileName(rplDirIndex)));
				}
				else {
					for (int i = 0; i < infoFields.length; i++) {
						infoFields[i] = "";
						gpInfo.updateValues(infoFields);
						lDirIndex.setText("0");
					}
					pConsole.appendToDump("No .RPL files in directory\n");
					bPrev.setEnabled(false);
					bNext.setEnabled(false);
					bRaceFromInfo.setEnabled(false);
				}

			}
		}
	};	

	private ActionListener onPrev = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			pConsole.setDumpField("");
			rplDirIndex--;
			if (rplDirIndex < 0)
				rplDirIndex = loadedDir.getRplCount() - 1; //"off by one" correction
			File f = new File(loadedDir.getDirPath() + File.separator + loadedDir.getFileName(rplDirIndex));
			loadRplFileInfo(f);
			lDirIndex.setText(Integer.toString(rplDirIndex + 1));
		}
	};	
	
	private ActionListener onNext = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			pConsole.setDumpField("");
			rplDirIndex++;
			if (rplDirIndex > loadedDir.getRplCount() - 1)
				rplDirIndex = 0;
			File f = new File(loadedDir.getDirPath() + File.separator + loadedDir.getFileName(rplDirIndex));
			loadRplFileInfo(f);
			lDirIndex.setText(Integer.toString(rplDirIndex + 1));
		}
	};	
	
	private ActionListener onRA = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			int[] fieldContents = new int[8];
			FrameWithClose frame = new FrameWithClose("JReplayAnalyzer");
			if (loadedReplay != null && loadedReplay.isValid())
				fieldContents = loadedReplay.logKeystrokes();
			for (int i = 0; i < raHeaders.length; i++) 
				raFields[i] = Integer.toString(fieldContents[i]);
			gpRA = new InfoGridPanel("Input Log", raHeaders, raFields, 20, 5);
			frame.mainBox().add(gpRA);
			TextDump raNotes = new TextDump("Information",
					"Count of input events. At 20fps, 20 events = 1s\n" +
		    		"Minimum count for \"No Event\" in a finished lap is 20\n" +
		    		"due to the final second after crossing the s/f line.");
		    frame.mainBox().add(raNotes);
		    frame.setVisible(true);

		}
	};
	

	private ActionListener[][] menuListeners = {
			{//file
				onLoad,
				onOpenDir,
				onExit
			},
			{//help
				onAbout
			}
	};
	
	//extends JFrame constructor
	public JRPLInfo(String string) {
		super(string);
	}
	
	private void loadRplFileInfo (File f) {
		String[] fieldContents;
		if (riFormatter == null)
			riFormatter = new InfoFormatter(f.getParentFile(), true);
		fieldContents = rplInfo(f);
		pConsole.appendToDump(fieldContents[0]);
		for (int i = INFO_TABLE_OFFSET; i < infoFields.length; i++)
			infoFields[i] = fieldContents[i-1];
		gpInfo.updateValues(infoFields);
		if (loadedReplay.isValid())
			bRA.setEnabled(true);
		else
			bRA.setEnabled(false);
		if (loadedDir != null && loadedDir.getRplCount() > 0) {
			rplDirIndex = loadedDir.searchRplList(f.getName());
			lDirIndex.setText(Integer.toString(rplDirIndex + 1));
		}
	}
	
	private String[] rplInfo (File f) {
		String[] fieldContents = new String[1 + MAX_INFO_FIELDS];
		rplPath = f.getPath();
		infoFields[0] = f.getName();
		infoFields[1] = f.getParent();
		if (isZSC)
			loadedReplay = new ZSCReplay(rplPath);
		else
			loadedReplay = new PlainReplay(rplPath);
		//the former rplInfo3 code
		fieldContents[RI3Fields.WARNINGS] = ""; //error messages
		try {
			if (loadedReplay.isValid()) {
				if (loadedReplay.newFormat())
					fieldContents[RI3Fields.VERSION] = "1.1";
				else
					fieldContents[RI3Fields.VERSION] = "1.0";
				fieldContents[RI3Fields.TRACK] = loadedReplay.getTrack();
				fieldContents[RI3Fields.CAR] = riFormatter.formatCarName(loadedReplay.getCar());
				//fieldContents[RI3Fields.COLOUR] = Integer.toString(loadedReplay.colourIndex());
				fieldContents[RI3Fields.COLOUR] = riFormatter.formatCarColour(loadedReplay.getCar(), loadedReplay.colourIndex());
				if (loadedReplay.autoTransmission())
					fieldContents[RI3Fields.TRANSM] = "Auto";
				else
					fieldContents[RI3Fields.TRANSM] = "Manual";
				int oppIndex = loadedReplay.opponentIndex();
				fieldContents[RI3Fields.OPP] = InfoFormatter.formatOpponentName(oppIndex);
				if (oppIndex != 0)
					fieldContents[RI3Fields.OPPCAR] = riFormatter.formatCarName(loadedReplay.opponentCarID());
				else
					fieldContents[RI3Fields.OPPCAR] = "";
				File trkFile = new File (loadedReplay.getRplFile().getParent(), loadedReplay.getTrack()+".TRK");
				try {
					if (loadedReplay.checkTrack(trkFile))
						fieldContents[RI3Fields.TRKOK] = "Yes";
					else
						fieldContents[RI3Fields.TRKOK] = "No";
				} catch (MissingReferenceTrackException e) {
					fieldContents[RI3Fields.WARNINGS] = fieldContents[RI3Fields.WARNINGS] + "No reference "+loadedReplay.getTrack()+".TRK available.\n";
				} 
				if (loadedReplay.checkTermination())
					fieldContents[RI3Fields.FINISH] = "Yes";
				else
					fieldContents[RI3Fields.FINISH] = "No";
				fieldContents[RI3Fields.DURATION] = 
					InfoFormatter.formatLaptime(Double.toString(loadedReplay.toSeconds(loadedReplay.computeDuration())));
				if (loadedReplay.getPenaltyDict() != null) {
					fieldContents[RI3Fields.TIME] = 
						InfoFormatter.formatLaptime(Double.toString(loadedReplay.toSeconds(loadedReplay.computeDuration(true)))); 
					if (!loadedReplay.isPenaltyMatched())
						fieldContents[RI3Fields.WARNINGS] = fieldContents[RI3Fields.WARNINGS] + "Penalty for this .rpl omitted, assuming 0.00s\n";
				}
				else
					fieldContents[RI3Fields.WARNINGS] = fieldContents[RI3Fields.WARNINGS] + "Local penalty.txt not found.\n";
				fieldContents[RI3Fields.CORRTIME] = 
					InfoFormatter.formatLaptime(Double.toString(loadedReplay.toSeconds(loadedReplay.computeTime())));
				fieldContents[RI3Fields.WARNINGS] = fieldContents[RI3Fields.WARNINGS] + loadedReplay.additionalStateWarnings();
			} 
			else 
				fieldContents[RI3Fields.WARNINGS] = fieldContents[RI3Fields.WARNINGS] + "This is not a valid .RPL!\n";
		} catch (FileNotFoundException e) {
			fieldContents[RI3Fields.WARNINGS] = fieldContents[RI3Fields.WARNINGS] + "Replay file not found!\n";
			System.err.println ("Replay file not found!");
			e.printStackTrace();
		} catch (IOException e) {
			fieldContents[RI3Fields.WARNINGS] = fieldContents[RI3Fields.WARNINGS] + "IO Exception on RPLInfo!\n";
			System.err.println ("IO exception on RPLInfo!");
			e.printStackTrace();
		}
		return fieldContents;	
	}
		
	private void loadRplDir (File dir) {
		riFormatter = new InfoFormatter(dir, true);
		loadedDir = new RplDirectory(dir);
		if (loadedDir != null)
			lDirCount.setText(Integer.toString(loadedDir.getRplCount()));
		else
			lDirCount.setText("0");
	}
	
	protected JMenuBar createMenuBar () {
		for (int i = 0; i < menus.length; i++)
			for (int j = 0; j < menuItems[i].length; j++) {
				menuItems[i][j].addActionListener(menuListeners[i][j]);
				menus[i].add(menuItems[i][j]);
			}
		JMenuBar mb = new JMenuBar();
		for(int i = 0; i < menus.length; i++)
			mb.add(menus[i]);
		return mb;
	}
	
	protected JPanel createRplInfoGUI () {
		JPanel pMain = new JPanel();
		pMain.setName("RPLInfo");
		pMain.setLayout(new BoxLayout(pMain, BoxLayout.Y_AXIS));
		
		gpInfo = new InfoGridPanel("Replay Information", infoHeaders, infoFields);
		pMain.add(gpInfo);
	    pConsole = new TextDump("Warnings", "");
	    pMain.add(pConsole);
	    
	    Box pRplFuncs = new Box(BoxLayout.X_AXIS);
		zscCheck.addActionListener(onZSCCheck);
		pRplFuncs.add(zscCheck);
		bRA.setEnabled(false);
		bRA.addActionListener(onRA);	
		pRplFuncs.add(bRA);
		pMain.add(pRplFuncs);
		
		bLoad.addActionListener(onLoad);
		bLoad.setAlignmentX(CENTER_ALIGNMENT);
		pMain.add(bLoad);
		Box pDirBox = new Box(BoxLayout.X_AXIS);
		bPrev.addActionListener(onPrev);
		bPrev.setEnabled(false);
		pDirBox.add(bPrev);
		pDirBox.add(Box.createHorizontalStrut(10));
		pDirBox.add(lDirIndex);
		pDirBox.add(new JLabel(" / "));
		pDirBox.add(lDirCount);
		bNext.addActionListener(onNext);
		bNext.setEnabled(false);
		pDirBox.add(Box.createHorizontalStrut(10));
		pDirBox.add(bNext);
		pMain.add(pDirBox);
		bOpenDir.addActionListener(onOpenDir);
		bOpenDir.setAlignmentX(CENTER_ALIGNMENT);
		pMain.add(bOpenDir);
		
		return pMain;
	}
	
	//Race creation variables
	
	Race currRace;
	Class<? extends BaseReplay> rplClass;
	
	//Race management GUI
	
	private JTextField tfRaceType = new JTextField("Race directory structure:");
	private JComboBox cbRaceType = new JComboBox();
	private String[] raceTypeChoices = {"All files in root directory",
			"All files in root dir, include all replays",
			"Root dir with racer subdirectories"};
	private int raceTypeIndex = 0;
	
	private JTextField tfReplayType = new JTextField("Replay parsing type:");
	private JComboBox cbReplayType = new JComboBox();
	private String[] replayTypeChoices = {"Plain replay",
			"ZSC replay (with bonuses)"};
	private int replayTypeIndex = 0;

	private JCheckBox checkPrintCorr = new JCheckBox("Print corrected times", true);
	private boolean printCorrected = true;
	
	private JCheckBox checkNoKeyLookup = new JCheckBox("Disable name key lookup", false);
	private boolean noKeyLookup = false;
	
	private JTextField tfNameKeyBoundsDesc = new JTextField("Char limits for racer IDs on filenames"),
						tfNameKeyStart = new JTextField("1"),
						tfNameKeyEnd = new JTextField("3");
	
	private int nameKeyStart = 0,
				nameKeyEnd = 3;
	
	private JButton bLoadRaceDir = new JButton("Load root dir");
	
	private ActionListener onLoadRaceDir = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			try {
				nameKeyStart = Integer.parseInt(tfNameKeyStart.getText());
				nameKeyEnd = Integer.parseInt(tfNameKeyEnd.getText());
			} catch (NumberFormatException ex) {
				nameKeyStart = 0;
				nameKeyEnd = 3;
			}
			JFileChooser c = new JFileChooser(workingDir);
			c.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			c.setDialogTitle("Open Directory");
			int rVal = c.showOpenDialog(JRPLInfo.this);
			if(rVal == JFileChooser.APPROVE_OPTION) {
				File d = c.getSelectedFile();
				workingDir = d; //changes workingDir even if dir is empty.
				createRaceAndScoreboard(d);
			}
		}
	};	
	
	private ActionListener onRaceType = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			raceTypeIndex = cbRaceType.getSelectedIndex();
			int[] typesWithKeyBounds = {0, 1};
			for (int i : typesWithKeyBounds) {
				if (raceTypeIndex == i) {
					tfNameKeyStart.setEditable(true);
					tfNameKeyEnd.setEditable(true);
					checkNoKeyLookup.setEnabled(true);
					break;
				}
				tfNameKeyStart.setEditable(false);
				tfNameKeyEnd.setEditable(false);
				checkNoKeyLookup.setEnabled(false);		
			}
		}
	};
	
	private ActionListener onReplayType = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			replayTypeIndex = cbReplayType.getSelectedIndex();
			int[] typesWithTimeCorr = {1};
			for (int i : typesWithTimeCorr) {
				if (replayTypeIndex == i) {
					checkPrintCorr.setEnabled(true);
					break;
				}
				checkPrintCorr.setEnabled(false);
			}
		}
	};
	
	private ActionListener onRaceFromInfo = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			tabs.setSelectedIndex(TAB_RACEVIEW);
			createRaceAndScoreboard(new File(loadedDir.getDirPath()));
		}
	};	
	
	private class ScoreboardFrame extends FrameWithClose {
		
		/**
		 * 
		 */
		private static final long serialVersionUID = 3575341097872579703L;
		
		private HIGTableComponent htc;
		
		public ScoreboardFrame (HIGTableComponent htc) {
			super("Race Scoreboard", "Close");
			setSize(640,480);
			this.htc = htc;
			addComponents();
			validate();
		}
		
		private void addComponents() {
			mainBox().add(htc);
			mainBox().revalidate();
		}

	}
	
	private void createRaceAndScoreboard (File d) {
		switch (replayTypeIndex) {
		case (0) : rplClass = PlainReplay.class; break;
		case (1) : rplClass = ZSCReplay.class; break;
		}
		switch (raceTypeIndex) {
		case (0) : currRace = FlatDirRace.create(d, rplClass, nameKeyStart - 1, nameKeyEnd, !noKeyLookup); break;
		case (1) : currRace = FlatDirReplaySet.create(d, rplClass, nameKeyStart - 1, nameKeyEnd, !noKeyLookup); break;
		case (2) : currRace = HierarchyRace.create(d, rplClass); break;
		}
		try {
		HIGTableComponent htc = new HIGTableComponent(currRace, printCorrected);
		ScoreboardFrame scoreFrame = new ScoreboardFrame(htc);
		scoreFrame.setVisible(true);
		} catch (NoScoreboardEntriesException e) {
			JOptionPane.showMessageDialog(this, "No valid scoreboard entries for this directory and race setup.");
		}
	}
	
	protected JPanel createRaceViewGUI () {
		JPanel pMain = new JPanel();
		pMain.setName("Race View");
		pMain.setLayout(new BoxLayout(pMain, BoxLayout.Y_AXIS));
		
		tfRaceType.setMaximumSize(new Dimension(320, 60));
		tfReplayType.setMaximumSize(new Dimension(320, 60));
		tfRaceType.setEditable(false);
		tfReplayType.setEditable(false);
		cbRaceType.setMaximumSize(new Dimension(320, 60));
		cbReplayType.setMaximumSize(new Dimension(320, 60));
		for (String choice : raceTypeChoices)
			cbRaceType.addItem(choice);
		cbRaceType.addActionListener(onRaceType);
		for (String choice : replayTypeChoices)
			cbReplayType.addItem(choice);
		cbReplayType.addActionListener(onReplayType);
	    pMain.add(tfRaceType);
	    pMain.add(cbRaceType);
	    pMain.add(tfReplayType);
	    pMain.add(cbReplayType);
	    pMain.add(Box.createVerticalGlue());
	    
		checkPrintCorr.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				printCorrected = !printCorrected;
			}
		});
		checkPrintCorr.setMaximumSize(new Dimension(320, 60));
		checkPrintCorr.setEnabled(false);
	    Box pPrintCorr = new Box(BoxLayout.X_AXIS);
	    pPrintCorr.setMaximumSize(new Dimension(320, 60));
	    pPrintCorr.add(checkPrintCorr);
		pMain.add(pPrintCorr);
		
		tfNameKeyBoundsDesc.setMaximumSize(new Dimension(320, 60));
		tfNameKeyBoundsDesc.setEditable(false);
		tfNameKeyStart.setEditable(false);
		tfNameKeyEnd.setEditable(false);
		pMain.add(tfNameKeyBoundsDesc);
	    Box pNameKey = new Box(BoxLayout.X_AXIS);
	    pNameKey.setMaximumSize(new Dimension(320, 60));
	    pNameKey.add(new JLabel("First"));
	    pNameKey.add(Box.createHorizontalStrut(10));
	    pNameKey.add(tfNameKeyStart);
	    pNameKey.add(Box.createHorizontalGlue());
	    pNameKey.add(new JLabel("Last"));
	    pNameKey.add(Box.createHorizontalStrut(10));
	    pNameKey.add(tfNameKeyEnd);
	    pMain.add(pNameKey);
		checkNoKeyLookup.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				noKeyLookup = !noKeyLookup;
			}
		});
		checkNoKeyLookup.setMaximumSize(new Dimension(320, 60));
		checkNoKeyLookup.setEnabled(false);
	    Box pNoKeyLookup = new Box(BoxLayout.X_AXIS);
	    pNoKeyLookup.setMaximumSize(new Dimension(320, 60));
	    pNoKeyLookup.add(checkNoKeyLookup);
	    pMain.add(pNoKeyLookup);
	    
	    pMain.add(Box.createVerticalGlue());
	    
	    bLoadRaceDir.addActionListener(onLoadRaceDir);
	    bLoadRaceDir.setAlignmentX(CENTER_ALIGNMENT);
	    pMain.add(bLoadRaceDir);
		
	    bRaceFromInfo.addActionListener(onRaceFromInfo);
	    bRaceFromInfo.setAlignmentX(CENTER_ALIGNMENT);
		bRaceFromInfo.setEnabled(false);
		pMain.add(bRaceFromInfo);
	    
		return pMain;
	}
	
	protected void createGUI () {
		JMenuBar mb = createMenuBar();
		setJMenuBar(mb);
		Container cpMain = getContentPane();
		tabs = new JTabbedPane();
		JPanel pRplInfo = createRplInfoGUI();
		tabs.add(pRplInfo, TAB_RPLINFO);
		JPanel pDirView = createRaceViewGUI();
		tabs.add(pDirView, TAB_RACEVIEW);
		cpMain.add(tabs);
	}
	
	public void init() {	
	    try {
	        javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
	            public void run() {
	                createGUI();
	            }
	        }
	        );
	        } catch (Exception e) { 
	        e.printStackTrace( );
	        System.err.println("createGUI didn't successfully complete");
	    }
	}
	
	public void stop() {
		Thread.currentThread().interrupt();
	}
	
	//JFrame main 
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.
					getSystemLookAndFeelClassName());
		} catch(Exception e) {
			throw new RuntimeException(e);
		}
		JRPLInfo frame;
		frame = new JRPLInfo("JRPLInfo 0.1");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(320,640);
		frame.init();
		frame.setVisible(true);
	}
	
	// JApplet main
	/*public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.
					getSystemLookAndFeelClassName());
		} catch(Exception e) {
			throw new RuntimeException(e);
		}
		JApplet applet;
		applet = new JRPLInfo();
		JFrame frame = new JFrame("JRPLInfo 0.1");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().add(applet);
		frame.setSize(320,640);
		applet.init();
		applet.start();
		frame.setVisible(true);
	}*/
}

