package hu.stunts.scr.apps;

import hu.stunts.scr.utils.enums.*;
import hu.stunts.scr.utils.TxtParsing;

import java.io.*;
import java.lang.Math;
import java.util.Map;

class ZSCReplay extends BaseReplay implements BonusTableReplay {
	
	private boolean isCarMatched = false;
	private double bonus = 0.0;
	private Map<String, String> bonusDict = null;

	// For creating a new penaltyDict instead of using an external object provide (Map<String, String>)null as second argument.
	public ZSCReplay(String fileName, Map<String, String>... dictionaries) {
		super(fileName, dictionaries);
		if (dictionaries.length > BaseReplayDicts.ZSCBONUS && dictionaries[BaseReplayDicts.ZSCBONUS] != null)
			bonusDict = dictionaries[BaseReplayDicts.ZSCBONUS];
		else
			initializeBonusDict();
		isCarMatched = readBonusDefault();
	}
	
	public ZSCReplay(String fileName) {
		this(fileName, (Map<String, String>)null);
	}
	
	private void initializeBonusDict() {
		int dictInitSize = 50;
		File bonusASCII = new File(getRplFile().getParent() + File.separator + "carbonus.txt");
		bonusDict = TxtParsing.simpleMakeMapFromPairs(bonusASCII, dictInitSize);
	}
	
	public final boolean readBonusDefault() {
		boolean found = false;
		double bon = 1.0;
		if (bonusDict != null) { //if the dict was properly initialized
			String val = bonusDict.get(getCar());
			if (val != null) {
				bon = Double.parseDouble(val);		
				found = true;
			}
		}
		bonus = (int)Math.round(bon);
		return found;
	}
	
	/* This method is meant to be used with an external bonus database,
	 * passed to the constructor, which gets updated during the lifetime
	 * of the replay.
	 */
	public void refreshBonus (){
		isCarMatched = readBonusDefault();
	}
	
	public double computeTime(boolean usePenalty) {
		return (((100 - bonus)/100)*computeDuration(usePenalty));
	}
	
	public double computeTime() {
		return computeTime(true);
	}
	
	public String[] rplInfo3() {
		String[] output = new String[MAXINFOFIELDS];
			output = super.rplInfo3();
			if (bonusDict != null) {
				if (isCarMatched)
					output[RI3Fields.CORRTIME] = Double.toString(toSeconds(computeTime()));
				else
					output[RI3Fields.WARNINGS] = output[RI3Fields.WARNINGS] + (getCar()+" bonus missing in carnames.txt.\n");
			}
			else 
				output[RI3Fields.WARNINGS] = output[RI3Fields.WARNINGS] + ("Missing carbonus.txt, ZSC correction skipped.\n");
		return output;
	}
	
	public String additionalStateWarnings() {
		String output = "";
		if (bonusDict != null) {
			if (!isCarMatched)
				output = output + getCar()+" bonus missing in carnames.txt.\n";
		}
		else 
			output = output + "Missing carbonus.txt, ZSC correction skipped.\n";
		return output;
	}
	
	public double getBonus() {
		return bonus;
	}
	
	public Map<String, String> getBonusDict() {
		return bonusDict;
	}
	
}
